@@ -1,3 +1,26 @@
+/***************************************************************************
+
+  dataset.h
+
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2, or (at your option)
+  any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
+
+***************************************************************************/
+
 /**********************************************************************
  * Copyright (c) 2002, Leo Seib, Hannover
  *
@@ -37,9 +60,10 @@
 #include <list>
 #include "qry_dat.h"
 
-extern "C" {
 #include "gambas.h"
 #include "gb_common.h"
+
+extern "C" {
 #include "../gb.db.h"
 
 extern GB_INTERFACE GB;
@@ -59,7 +83,7 @@ class Dataset;		// forward declaration of class Dataset
 #define DB_COMMAND_OK		0	// OK - command executed
 #define DB_EMPTY_QUERY		1	// Query didn't return tuples
 #define DB_TUPLES_OK		2	// Query returned tuples
-#define DB_ERROR		5
+#define DB_GOT_ERROR		5
 #define DB_BAD_RESPONSE		6
 #define DB_UNEXPECTED		7	// This shouldn't ever happen
 #define DB_UNEXPECTED_RESULT   -1       //For integer functions
