@@ -1249,21 +1249,15 @@ static void load_without_inits(CLASS *class)
 
 	// JIT compilation
 
-	for(i = 0; i < class->load->n_func; i++)
+	for (i = 0; i < class->load->n_func; i++)
 	{
 		if (class->load->func[i].fast)
 		{
 			ARCHIVE *arch = class->component ? class->component->archive : NULL;
 			
 			if (JIT_can_compile(arch))
-			{
-				if (JIT_compile(arch))
-				{
-					for(i = 0; i < class->load->n_func; i++)
-						class->load->func[i].fast = FALSE;
-				}
-			}
-			
+				JIT_compile(arch);
+
 			break;
 		}
 	}
@@ -1339,6 +1333,9 @@ void CLASS_load_real(CLASS *class)
 		}
 	}
 
+	/*if (strcmp(class->name, "Project") == 0)
+		BREAKPOINT();*/
+	
 	load_without_inits(class);
 	class->loaded = TRUE;
 	class->ready = FALSE;
