@@ -43,8 +43,9 @@ bool JIT_disabled = FALSE;
 
 static bool _component_loaded = FALSE;
 static GB_FUNCTION _jit_compile_func;
+static GB_FUNCTION _jit_wait_func;
 
-static bool _jit_compiling = FALSE;
+static char _jit_state = JIT_NOT_COMPILED;
 static void *_jit_library = NULL;
 
 static JIT_FUNCTION *_jit_func = NULL;
@@ -56,32 +57,38 @@ void JIT_exit(void)
 	ARRAY_delete(&_jit_func);
 }
 
-bool JIT_can_compile(ARCHIVE *arch)
+void JIT_abort(void)
 {
-	return arch ? !arch->jit_compiling : !_jit_compiling;
+	static GB_FUNCTION _func;
+	
+	if (!_component_loaded || JIT_disabled)
+		return;
+	
+	if (GB_GetFunction(&_func, CLASS_find_global("Jit"), "_Abort", NULL, NULL))
+		ERROR_panic("Unable to find JIT._Abort() method");
+	
+	GB_Call(&_func, 0, FALSE);
 }
 
-bool JIT_compile(ARCHIVE *arch)
+static int get_state(ARCHIVE *arch)
+{
+	return arch ? arch->jit_state : _jit_state;
+}
+
+bool JIT_can_compile(ARCHIVE *arch)
+{
+	return get_state(arch) == JIT_NOT_COMPILED;
+}
+
+void JIT_compile(ARCHIVE *arch)
 {
-	GB_VALUE *ret;
-	char *path;
-	void *lib;
-	void **iface;
 	COMPONENT *current;
 	
 	if (JIT_disabled)
-		return TRUE;
+		return;
 	
-	if (arch)
-	{
-		if (arch->jit_library)
-			return FALSE;
-	}
-	else
-	{
-		if (_jit_library)
-			return FALSE;
-	}
+	if (arch ? arch->jit_library : _jit_library)
+		return;
 	
 	if (!_component_loaded)
 	{
@@ -93,7 +100,7 @@ bool JIT_compile(ARCHIVE *arch)
 		if (var && var[0] && !(var[0] == '0' && var[1] == 0))
 		{
 			JIT_disabled = TRUE;
-			return TRUE;
+			return;
 		}
 		
 		var = getenv("GB_JIT_DEBUG");
@@ -104,26 +111,51 @@ bool JIT_compile(ARCHIVE *arch)
 			fprintf(stderr, "gbx3: loading gb.jit component\n");
 		
 		COMPONENT_load(COMPONENT_create("gb.jit"));
-		if (GB_GetFunction(&_jit_compile_func, CLASS_find_global("Jit"), "_Compile", "s", "s"))
-			ERROR_panic("Unable to find JIT compilation method");
+		if (GB_GetFunction(&_jit_compile_func, CLASS_find_global("Jit"), "_Compile", "s", "b"))
+			ERROR_panic("Unable to find JIT._Compile() method");
+		if (GB_GetFunction(&_jit_wait_func, CLASS_find_global("Jit"), "_Wait", "s", "s"))
+			ERROR_panic("Unable to find JIT._Wait() method");
 	}
 	
-	arch ? (arch->jit_compiling = TRUE) : (_jit_compiling = TRUE);
+	arch ? (arch->jit_state = JIT_COMPILING) : (_jit_state = JIT_COMPILING);
 	
 	current = COMPONENT_current;
 	COMPONENT_current = NULL;
 	
 	GB_Push(1, T_STRING, arch ? arch->name : "", -1);
-	ret = GB_Call(&_jit_compile_func, 1, FALSE);
-	path = GB_ToZeroString((GB_STRING *)ret);
+	GB_Call(&_jit_compile_func, 1, FALSE);
+	
+	COMPONENT_current = current;
+}
+
+bool wait_for_compilation(ARCHIVE *arch)
+{
+	COMPONENT *current;
+	void *lib;
+	void **iface;
+	GB_VALUE *ret;
+	char *path;
+
+	if (JIT_disabled)
+		return TRUE;
+	
+	if (arch ? arch->jit_library : _jit_library)
+		return FALSE;
+	
+	current = COMPONENT_current;
+	COMPONENT_current = NULL;
+	
+	GB_Push(1, T_STRING, arch ? arch->name : "", -1);
+	ret = GB_Call(&_jit_wait_func, 1, FALSE);
 	
 	COMPONENT_current = current;
 	
+	arch ? (arch->jit_state = JIT_COMPILED) : (_jit_state = JIT_COMPILED);
+	
+	path = GB_ToZeroString((GB_STRING *)ret);
 	if (!*path)
 		ERROR_panic("Unable to compile JIT source file");
-	
-	arch ? (arch->jit_compiling = FALSE) : (_jit_compiling = FALSE);
-	
+
 	//fprintf(stderr, "gbx3: shared jit library is: %s\n", path);
 
 	lib = dlopen(path, RTLD_NOW);
@@ -157,6 +189,19 @@ static bool create_function(CLASS *class, int index)
 	
 	arch = class->component ? class->component->archive : NULL;
 	
+	if (!class->loaded)
+		return TRUE;
+	
+	/*if (get_state(arch) == JIT_NOT_COMPILED) // don't use JIT if we are compiling the corresponding archive
+		return TRUE;*/
+	
+	if (wait_for_compilation(arch))
+	{
+		for (i = 0; i < class->load->n_func; i++)
+			class->load->func[i].fast = FALSE;
+		return TRUE;
+	}
+	
 	func = &class->load->func[index];
 	func->fast_linked = TRUE;
 	
@@ -198,11 +243,12 @@ static bool create_function(CLASS *class, int index)
 }
 
 
-void JIT_exec(bool ret_on_stack)
+bool JIT_exec(bool ret_on_stack)
 {
 	VALUE *sp = SP;
 	JIT_FUNCTION *jit;
 	CLASS *class = EXEC.class;
+	void *object = EXEC.object;
 	char nparam = EXEC.nparam;
 	VALUE ret;
 	FUNCTION *func = EXEC.func;
@@ -215,13 +261,13 @@ void JIT_exec(bool ret_on_stack)
 	if (!func->fast_linked)
 	{
 		if (create_function(class, EXEC.index))
-			return;
+			return TRUE;
 	}
 	
 	STACK_push_frame(&EXEC_current, func->stack_usage);
 	
 	CP = class;
-	OP = (void *)EXEC.object;
+	OP = object;
 	FP = func;
 	EC = NULL;
 	
@@ -274,6 +320,8 @@ void JIT_exec(bool ret_on_stack)
 		*SP++ = ret;
 		ret.type = T_VOID;
 	}
+	
+	return FALSE;
 }
 
 PCODE *JIT_get_code(FUNCTION *func)
