@@ -166,7 +166,7 @@ typedef
 #define PARAM_OPT_s(_p) PARAM_OPT(_p, s, GET_CSTRING("", 0, 0))
 #define PARAM_OPT_o(_p) PARAM_OPT(_p, o, GET_OBJECT(0, GB_T_OBJECT))
 #define PARAM_OPT_v(_p) PARAM_OPT(_p, v, ({ GB_VARIANT _v; _v.type = GB_T_VARIANT; _v.value.type = GB_T_NULL; _v; }))
-#define PARAM_OPT_O(_p, _type) (((_p) >= n || (sp[-n+(_p)].type == GB_T_VOID)) ? GET_OBJECT(GB_T_OBJECT, 0) : PARAM_O(_p, _type))
+#define PARAM_OPT_O(_p, _type) (((_p) >= n || (sp[-n+(_p)].type == GB_T_VOID)) ? GET_OBJECT(0, GB_T_OBJECT) : PARAM_O(_p, _type))
 
 #define OPT(_p, _n) ({ \
   uchar _opt = 0; \
@@ -427,6 +427,8 @@ enum
 
 #define CONV_o_O(_val, _class) CONV(_val, o, o, CLASS(_class))
 
+#define CONV_o(_val) ({ GB_OBJECT temp = (_val); temp.type = GB_T_OBJECT; temp; })
+
 #define GET_NULL_o() ({ GB_OBJECT temp; temp.type = GB_T_OBJECT; temp.value = NULL; temp; })
 #define GET_NULL_v() ({ GB_VARIANT temp; temp.type = GB_T_VARIANT; temp.value.type = GB_T_NULL; temp; })
 #define GET_NULL_s() GET_CSTRING("", 0, 0)
